/*
  bb_test.c byt SHDesigns http://shdesigns.org/tips/

  This sample uses bb_ram.lib to allocate a battery-backed near RAM segment.

  For boards with FAST RAM (RCM32xx, RCM33xx, RCM38xx), define FAST_RAM as 1
  For boards without the FAST RAM define FAST_RAM as 0

  for FAST_RAM=1, set up the linker with

  BB_BSS	- before BSS, set tp "Precede following segment",
  				 type Data, alignment 1000
  BSS		- change alignment to 1000
  BSSZ 		- unchanged
  IDATA		- unchanged
  STACK		- unchanged

  for FAST RAM=0, set up the linker with

  BSS		- unchanged
  BB_BSS	- before BSSZ, set tp "Precede following segment",
  				 type Data
  BSSZ 		- unchanged
  IDATA		- unchanged
  STACK		- unchanged


  Add bb_ram.lib to the link Lib and obj list.

  The BB_BSS seg is forced to a multiple of 4096. You can minimize near
  wasted space by making it as close to a multiple of 4096.

  This sample uses a BB_KEY to test if memory is valid. An option may
  be to use CRC on critical data to detect for corruction. A "key"
  variable is usually good enough.

  Notes:
	This test changes BBRAM to use 4 wait states. This is only
	to verify the RAM is actually being used. Speed tests will show
	the difference in speed if the mapping is correct. On boards without
	FAST RAM, ther is no difference.

	There is one reason to use the map_bbram() on boards without fast RAM:
	the battery-backed variables will be put in the top of memory. They will
	not be corrupted by the IDE.


*/


#include <stdio.h>
#include <rabbit.h>
#include <string.h>

#include "..\bb_ram.h"

#define FAST_RAM 1	// set to 0 for boards without fast RAM
#define BB_KEY 0xFACE55AAl

const char test_string[]="This is test to see of data is maintained in BB_RAM.\n";

#pragma SEG(BSS,BB_BSS)
// all your BB variables go here.
char bb_buffer[2048];
long bb_longs[1500];
unsigned long bb_test;
#pragma SEG(BSS)

long fr_longs[1500];	// fast RAM data

unsigned long speed_test(long * buff,unsigned size)
{
	long * tbuff;
	unsigned tsize;
	unsigned long stime=getMilliSeconds( );
	for (int i=0;i<500;i++)
	{
		tbuff=buff;
		tsize=size;
		while (tsize--)
		{
			(*tbuff)++;
			tbuff++;
		}
	}
	return getMilliSeconds( )-stime;
}

void displayMBxCR( unsigned cr, unsigned sr )
{
static const char ws[] = { 4, 2, 1, 0 };

	printf( "  MB%dCR: CS%d/OE%d, WE: %s, %d rd: %d wr wait states.\r\n",
		cr, sr & 3, (srMB0CR & 4) != 0, (sr & 8) == 0 ? "Yes" : "No",
		ws[ sr >> 6 ], ws[ sr >> 6 ] + 1 );
}

extern char far *_xmemEnd;	// used by mem info
void show_meminfo(void)
{
	char *x = (char *) 0xDFFF;
	printf( "\x1A""\r\n\n\r\nRunning in: %s\r\nMemory Bank Info:\r\n", _inFlash() ? "flash" : "RAM" );
	displayMBxCR( 0, srMB0CR );
	displayMBxCR( 1, srMB1CR );
	displayMBxCR( 2, srMB2CR );
	displayMBxCR( 3, srMB3CR );

	printf( "\r\nXmem Info:\r\n" );
	printf( "  Stack end : %lX\r\n", (char far *) x );
	printf( "  Xmem base : %lX\r\n", xalloc( 0 ) );
	printf( "  Xmem free : %lX (%uk)\r\n", xavail(), xavail() / 1024 );
	printf( "  Xmem end  : %lX\r\n", _xmemEnd - 1 );
}

void main(void)
{
	unsigned long bb_size;
	char far * bb_base;
	char far * bb_array;
	ipset0();
	WDT_DISABLE();
	startTimer(10,NULL,1);
#if FAST_RAM
	// map upper RAM with lots of waits tates, that way we can
	// tell be the difference in speed
    // you do NOT want to do this, this is just for testing!
	outi(MB2CR,srMB2CR&=0x3f);
	outi(MB3CR,srMB3CR&=0x3f);
#endif

    printf("Startup.\n");
	show_meminfo();

#if FAST_RAM
// if running in FAST RAM, we need to map the BB_BSS segment.
	bb_size=get_bb_size();	// ask lib size of BB_BSS
    // assume this works
	bb_base=bb_xalloc(bb_size); // you should see the test string after the first run
    map_bb_ram(bb_base);  // map DATASEG to BBRAM and STACSEG to fast RAM
#endif
	// allocate a BB_RAM far array
    bb_array=bb_xalloc(1234);	// allocate an array in BB_RAM
	printf("\nbb_Array is at 0x%lX\n",bb_array);
	// test flasg to see if mem is valid
	if (bb_test!=BB_KEY)
	{
		// init variables because flag is invalid
		printf("Key not valid, must be first run, init variables.\n");
		strcpy(bb_buffer,test_string);
		bb_test=BB_KEY;
	}
	else
		printf("Memory key is valid ;)\n");
	if (strcmp(bb_buffer,test_string)==0)
		printf("Test string is valid.\n");
	else
	{
		printf("Test string is corrupted.\n");
		strcpy(bb_buffer,test_string);
	}
	// test speed of RAm to see differences
	printf("Speed test of fast RAM:   %ld ms.\n",speed_test(fr_longs, sizeof( fr_longs)/4));
	printf("Speed test of BB_BSS RAM: %ld ms.\n",speed_test(bb_longs, sizeof( bb_longs)/4));
	while (1)
	{

	}
}
